@@ -16,6 +16,7 @@ import tempfile
 import unittest
 
 from genshi.builder import tag
+from genshi.core import Markup
 import trac.tests.compat
 from trac.core import Component, TracError, implements
 from trac.test import EnvironmentStub, locale_en
@@ -41,13 +42,10 @@ class ChromeTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub()
-        from trac.core import ComponentMeta
-        self._old_registry = ComponentMeta._registry
-        ComponentMeta._registry = {}
+        self.env.clear_component_registry()
 
     def tearDown(self):
-        from trac.core import ComponentMeta
-        ComponentMeta._registry = self._old_registry
+        self.env.restore_component_registry()
 
     def _get_navigation_item(self, items, name):
         for item in items:
@@ -175,10 +173,12 @@ class ChromeTestCase(unittest.TestCase):
         add_fn(req, 'Message with an "&"')
         add_fn(req, Exception("Exception message with an &"))
         add_fn(req, tag("Message with text ", tag.b("& markup")))
+        add_fn(req, Markup("Markup <strong>message</strong>."))
         messages = req.chrome[msgtype]
         self.assertIn('Message with an "&amp;"', messages)
         self.assertIn("Exception message with an &amp;", messages)
         self.assertIn("Message with text <b>&amp; markup</b>", messages)
+        self.assertIn("Markup <strong>message</strong>.", messages)
 
     def test_add_warning_escapes_markup(self):
         """Message text is escaped. Regression test for
@@ -366,6 +366,40 @@ class ChromeTestCase(unittest.TestCase):
         self.assertEqual('test1', items[0]['name'])
         self.assertEqual('test2', items[1]['name'])
 
+    def test_add_jquery_ui_first_week_day(self):
+        def first_week_day(locale, lc_time, languages):
+            chrome = Chrome(self.env)
+            req = Request(href=Href('/trac.cgi'), locale=locale,
+                          lc_time=lc_time, tz=utc, languages=languages)
+            chrome.add_jquery_ui(req)
+            return req.chrome['script_data']['jquery_ui']['first_week_day']
+
+        # Babel is unavailable
+        self.assertEqual(0, first_week_day(None, None, None))
+        self.assertEqual(1, first_week_day(None, 'iso8601', None))
+        if locale_en:
+            # We expect the following aliases
+            from babel.core import LOCALE_ALIASES, Locale
+            self.assertEqual('ja_JP', LOCALE_ALIASES['ja'])
+            self.assertEqual('de_DE', LOCALE_ALIASES['de'])
+            self.assertEqual('fr_FR', LOCALE_ALIASES['fr'])
+
+            self.assertEqual(0, first_week_day(locale_en, locale_en, []))
+            self.assertEqual(1, first_week_day(locale_en, 'iso8601', []))
+            ja = Locale.parse('ja')
+            self.assertEqual(0, first_week_day(ja, ja, []))
+            self.assertEqual(0, first_week_day(ja, ja, ['ja', 'ja-jp']))
+            de = Locale.parse('de')
+            self.assertEqual(1, first_week_day(de, de, []))
+            self.assertEqual(1, first_week_day(de, de, ['de', 'de-de']))
+            fr = Locale.parse('fr')
+            self.assertEqual(1, first_week_day(fr, fr, []))
+            self.assertEqual(1, first_week_day(fr, fr, ['fr', 'fr-fr']))
+            self.assertEqual(0, first_week_day(fr, fr, ['fr', 'fr-ca']))
+            # invalid locale identifier (#12408)
+            self.assertEqual(1, first_week_day(fr, fr, ['fr', 'fr-']))
+            self.assertEqual(0, first_week_day(fr, fr, ['fr', 'fr-', 'fr-ca']))
+
     def test_add_jquery_ui_timezone_list_has_default_timezone(self):
         chrome = Chrome(self.env)
         href = Href('/trac.cgi')
